import 'package:docker_cli/src/container.dart';

import 'containers.dart';
import 'docker.dart';
import 'exceptions.dart';
import 'image_name.dart';

class Image {
  final ImageName _imageName;

  final String? imageid;
  final String? created;
  final String? size;

  Image(
      {required String repositoryAndName,
      required String tag,
      required this.imageid,
      required this.created,
      required this.size})
      : _imageName =
            ImageName.fromRepositoryAndName(repositoryAndName, tag: tag);

  Image.fromName(String fullname)
      : imageid = null,
        created = null,
        size = null,
        _imageName = ImageName.fromName(fullname);

  String get fullname => _imageName.fullname;

  String get name => _imageName.name;

  String? get repository => _imageName.repository;

  String? get tag => _imageName.tag;

  /// deletes this docker image.
  void delete() {
    dockerRun('image', 'rm $imageid');
  }

  /// Pulls a docker image from a remote repository using the
  /// images [fullname]
  void pull() {
    dockerRun('pull', fullname);
  }

  /// creates a container with the name [containerName] using
  /// this image.
  /// Returns the newly created [Container].
  /// Throws a [ContainerExistsException] if a container with
  /// the passed [containerName] already exists.
  Container create(String containerName,
      {List<String>? args, String? argString}) {
    if (Containers().findByName(containerName) != null) {
      throw ContainerExistsException(containerName);
    }

    var cmdArgs = '--name $containerName $fullname';

    if (args != null) {
      cmdArgs += ' ' + args.join(' ');
    }
    if (argString != null) {
      cmdArgs += ' ' + argString;
    }

    var lines = dockerRun('create', cmdArgs);

    var containerid = lines[0];

    var container = Containers().findByContainerId(containerid);

    if (container == null) {
      throw ContainerNotFoundException();
    }
    return container;
  }

  /// Returns true if the pass name components match
  /// this image.
  /// This method allows you to do a partial match by
  /// passing only the components you want to match on.
  /// The [name] must be passed.
  bool isSame({String? repository, required String name, String? tag}) {
    if (repository != null) {
      if (this.repository != repository) return false;
    }

    if (this.name != name) return false;

    if (tag != null) {
      if (this.tag != tag) return false;
    }

    return true;
  }

  @override
  bool operator ==(covariant Image image) {
    return imageid == image.imageid;
  }

  @override
  int get hashCode => imageid.hashCode;
}
